module Tweed
  module Monads
    ## The Maybe monad
    class Maybe < Tweed::Monad
      define_monad do
        construct { |value, success| @value, @success = value, success }
        unit { |value| self.new(value, true) }
        bind { |&block| @success ? block.call(@value) : self }
        zero { self.new(nil, false) }
      end
      
      def inspect
        repr = @success ? "[#{@value.inspect}]" : ".zero"
        "#{self.class.inspect}#{repr}"
      end
      
      def ==(other)
        if other.is_a? Maybe
          [@value, @success] == other.instance_eval { [@value, @success] }
        else
          self.object_id == other.object_id || @value == other
        end
      end
      
      # An example of safe division implemented over the Maybe monad.
      module SafeDivision
        def /(other)
          self.bind do |x|
            other.bind do |y|
              y.zero? ? self.class.zero : self.class[x / y]
            end
          end
        end # def /
      end # module SafeDivision
    end # class Maybe
  end # module Monads
end # module Tweed